﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Net;
using System.Threading;
using System.ServiceModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

using AgentTabSl.RgsAgentService;

namespace AgentTabSl
{
    public partial class Page : UserControl
    {
        private ProxyServiceSoapClient _service;

        private Timer _timer;

        public Page()
        {
            InitializeComponent();
        }

        private void OnLoaded(object sender, RoutedEventArgs e)
        {
            _service = new ProxyServiceSoapClient();

            _service.Endpoint.Address = new EndpointAddress("http://localhost:55984/Rgs/Clients/ProxyService.asmx");

            _service.GetGroupsCompleted += OnGetGroupsCompleted;
            _service.GetAgentCompleted += OnGetAgentCompleted;
            _service.SignInCompleted += OnSignInCompleted;
            _service.SignOutCompleted += OnSignOutCompleted;

            _service.GetAgentAsync();

            _timer = new Timer(OnTimer, null, 0, 60 * 1000);
        }

        private void OnGetAgentCompleted(object sender, GetAgentCompletedEventArgs e)
        {
            Agent.Text = e.Result.SipAddress;
        }

        private void OnGetGroupsCompleted(object sender, GetGroupsCompletedEventArgs e)
        {
            if (!Dispatcher.CheckAccess())
            {
                Dispatcher.BeginInvoke(() => OnGetGroupsCompleted(sender, e));
            }
            else
            {
                IEnumerable<AcdGroup> groups = e.Result.OrderBy(group => group.Name);

                Groups.Children.Clear();

                foreach (AcdGroup group in groups)
                {
                    Group ctrl = Group.Create(group.Id, group.CanSignIn, group.IsSignedIn,
                                              group.Name, group.NumberOfAgents);

                    ctrl.SignedInChanged += OnSignedInChanged;

                    Groups.Children.Add(ctrl);
                }

                SetStatus("Ready.");
            }
        }

        private void OnSignInCompleted(object sender, SignInCompletedEventArgs e)
        {
            SetStatus("Ready.");
        }

        private void OnSignOutCompleted(object sender, SignOutCompletedEventArgs e)
        {
            SetStatus("Ready.");
        }

        private void OnSignedInChanged(object sender, EventArgs e)
        {
            Group group = sender as Group;

            SetStatus("Please wait while the operation is performed ...");

            if (group.IsSignedIn)
            {
                _service.SignInAsync(group.Id);
            }
            else
            {
                _service.SignOutAsync(group.Id);
            }
        }

        private void OnTimer(object state)
        {
            SetStatus("Please wait while the data is refreshed ...");

            RequestData();
        }

        private void RequestData()
        {
            _service.GetGroupsAsync();
        }

        private void SetStatus(string message)
        {
            if (!Dispatcher.CheckAccess())
            {
                Dispatcher.BeginInvoke(() => SetStatus(message));
            }
            else
            {
                Status.Text = message;
            }
        }
    }
}
